.recorder-time-label {
  font-size: 32pt;
  font-weight: 300
}

@keyframes clocks-blink {
	0% { color: @theme_text_color; }
	100% { color: #FF0000; }
}

.recorder-time-label.paused {
	color: #FF0000;
	animation-name: clocks-blink;
	animation-iteration-count: infinite;
	animation-timing-function: cubic-bezier(1.0,0,0,1.0);
	animation-duration: 1s;
}

button.rounded-button {
  padding: 4px;
  border-radius: 9999px;
	-gtk-outline-radius: 9999px;
  min-width: 24px;
  min-height: 24px;
}

button.large {
  padding: 12px;
}

list row label.title {
  font-weight: bold;
}

list row label.subtitle {
  font-size: smaller;
}

list row {
  margin-top: 0;
  margin-bottom: 0;
  border-radius: 0;
}

list row:first-child {
  border-top-left-radius: 8px;
  border-top-right-radius: 8px;
}

list row:last-child {
  border-bottom-left-radius: 8px;
  border-bottom-right-radius: 8px;
}

list row.expanded {
  margin-top: 10px;
  margin-bottom: 10px;
  border-radius: 8px;
  border-bottom-width: 1px;
  transition: 250ms all linear;
}

list row.expanded-before {
  border-bottom-width: 1px;
  border-bottom-right-radius: 8px;
  border-bottom-left-radius: 8px;
  transition: 250ms all linear;
}

list row.expanded-after {
  border-top-right-radius: 8px;
  border-top-left-radius: 8px;
  transition: 250ms all linear;
}

list row:first-child.expanded {
  margin-top:0;
}

list row:last-child.expanded {
  margin-bottom: 0;
}
